# -*- coding: utf-8 -*-
import sys
import os
import sys

# Force X11 backend (xcb) on Linux to support XShape click-through
# This must be set before QApplication is instantiated.
if sys.platform == "linux":
    os.environ["QT_QPA_PLATFORM"] = "xcb"

from PyQt6.QtWidgets import QApplication
import qasync
import asyncio
from .danmaku_widget import DanmakuWidget


async def main(app, room_id: int = 7450109):
    app_close_event = asyncio.Event()
    app.aboutToQuit.connect(app_close_event.set)

    # 创建弹幕窗口
    danmaku_widget = DanmakuWidget(room_id)

    # 显示窗口
    danmaku_widget.show()

    await app_close_event.wait()

def entry_point():
    import argparse
    from PyQt6.QtCore import Qt

    parser = argparse.ArgumentParser(description="B站弹幕阅读器")
    parser.add_argument("--room-id", "-r", type=int, default=7450109, help="直播间ID")
    args = parser.parse_args()

    os.environ["QT_AUTO_SCREEN_SCALE_FACTOR"] = "0"
    os.environ["QT_SCALE_FACTOR"] = "1"

    if hasattr(Qt.HighDpiScaleFactorRoundingPolicy, 'PassThrough'):
        QApplication.setHighDpiScaleFactorRoundingPolicy(
            Qt.HighDpiScaleFactorRoundingPolicy.PassThrough
        )
    
    app = QApplication(sys.argv)
    
    loop = qasync.QEventLoop(app)
    asyncio.set_event_loop(loop)
    
    try:
        loop.run_until_complete(main(app, args.room_id))
    finally:
        loop.close()

if __name__ == "__main__":
    entry_point()



# -*- coding: utf-8 -*-
from PyQt6.QtWidgets import (
    QDialog, QVBoxLayout, QLabel, QPushButton, QWidget, 
    QGraphicsDropShadowEffect
)
from PyQt6.QtCore import QTimer, Qt, pyqtSignal, QSize
from PyQt6.QtGui import QPixmap, QImage, QPainter, QColor
import qasync
import asyncio
from .auth import AuthManager

class QRLoginDialog(QDialog):
    login_success = pyqtSignal()
    
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("扫码登录 Bilibili")
        self.setFixedSize(320, 400)
        
        # Modern window styling
        self.setWindowFlags(Qt.WindowType.Dialog | Qt.WindowType.FramelessWindowHint)
        self.setAttribute(Qt.WidgetAttribute.WA_TranslucentBackground)
        
        self.auth_manager = AuthManager()
        self.qrcode_key = None
        
        self.init_ui()
        
        # Timer for polling
        self.poll_timer = QTimer(self)
        self.poll_timer.setInterval(2000) # Poll every 2 seconds
        self.poll_timer.timeout.connect(self.check_status)
        
    def init_ui(self):
        # Main layout with background container
        main_layout = QVBoxLayout(self)
        main_layout.setContentsMargins(10, 10, 10, 10)
        
        self.container = QWidget()
        self.container.setStyleSheet("""
            QWidget {
                background-color: #2b2b2b;
                border-radius: 12px;
                border: 1px solid #3d3d3d;
            }
        """)
        
        # Add shadow
        shadow = QGraphicsDropShadowEffect(self)
        shadow.setBlurRadius(20)
        shadow.setColor(QColor(0, 0, 0, 100))
        self.container.setGraphicsEffect(shadow)
        
        layout = QVBoxLayout(self.container)
        layout.setSpacing(15)
        layout.setContentsMargins(20, 20, 20, 20)
        
        # Title
        title_label = QLabel("扫码登录")
        title_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        title_label.setStyleSheet("""
            font-size: 18px;
            font-weight: bold;
            color: #ffffff;
            font-family: 'Microsoft YaHei';
        """)
        layout.addWidget(title_label)
        
        # QR Code Display
        self.qr_label = QLabel()
        self.qr_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.qr_label.setFixedSize(200, 200)
        self.qr_label.setStyleSheet("background-color: white; border-radius: 4px;")
        layout.addWidget(self.qr_label, 0, Qt.AlignmentFlag.AlignCenter)
        
        # Status Label
        self.status_label = QLabel("正在加载二维码...")
        self.status_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.status_label.setWordWrap(True)
        self.status_label.setStyleSheet("""
            font-size: 14px;
            color: #aaaaaa;
            font-family: 'Microsoft YaHei';
        """)
        layout.addWidget(self.status_label)
        
        # Refresh Button
        self.refresh_btn = QPushButton("刷新二维码")
        self.refresh_btn.setCursor(Qt.CursorShape.PointingHandCursor)
        self.refresh_btn.setVisible(False)
        self.refresh_btn.setStyleSheet("""
            QPushButton {
                background-color: #00a1d6;
                color: white;
                border: none;
                border-radius: 4px;
                padding: 8px 16px;
                font-size: 14px;
            }
            QPushButton:hover {
                background-color: #00b5e5;
            }
        """)
        self.refresh_btn.clicked.connect(self.refresh_qrcode)
        layout.addWidget(self.refresh_btn, 0, Qt.AlignmentFlag.AlignCenter)

        # Close Button
        close_btn = QPushButton("取消")
        close_btn.setCursor(Qt.CursorShape.PointingHandCursor)
        close_btn.setStyleSheet("""
            QPushButton {
                background-color: transparent;
                color: #888888;
                border: none;
                font-size: 13px;
            }
            QPushButton:hover {
                color: #ffffff;
                text-decoration: underline;
            }
        """)
        close_btn.clicked.connect(self.close)
        layout.addWidget(close_btn, 0, Qt.AlignmentFlag.AlignCenter)
        
        main_layout.addWidget(self.container)

    def showEvent(self, event):
        super().showEvent(event)
        self.refresh_qrcode()
        
    def closeEvent(self, event):
        self.poll_timer.stop()
        super().closeEvent(event)
        
    def refresh_qrcode(self):
        self.status_label.setText("正在获取二维码...")
        self.status_label.setStyleSheet("color: #aaaaaa;")
        self.refresh_btn.setVisible(False)
        self.poll_timer.stop()
        
        # Async call to get QR code
        asyncio.create_task(self._load_qrcode())
        
    async def _load_qrcode(self):
        url, key = await self.auth_manager.get_qrcode()
        if url and key:
            self.qrcode_key = key
            
            # Generate Image
            # Note: generate_qr_image is synchronous but fast
            loop = asyncio.get_event_loop()
            bio = await loop.run_in_executor(None, self.auth_manager.generate_qr_image, url)
            
            if bio:
                top_img = QImage.fromData(bio.getvalue())
                pixmap = QPixmap.fromImage(top_img)
                self.qr_label.setPixmap(pixmap.scaled(180, 180, Qt.AspectRatioMode.KeepAspectRatio, Qt.TransformationMode.SmoothTransformation))
                self.status_label.setText("请使用 哔哩哔哩手机客户端 扫码")
                self.poll_timer.start()
            else:
                self.status_label.setText("生成二维码失败")
                self.refresh_btn.setVisible(True)
        else:
            self.status_label.setText("无法连接到服务器")
            self.refresh_btn.setVisible(True)

    def check_status(self):
        if not self.qrcode_key:
            return
        asyncio.create_task(self._poll_status())
            
    async def _poll_status(self):
        code, msg, cookies = await self.auth_manager.poll_status(self.qrcode_key)
        print(f"Poll Status: {code}, Msg: {msg}") 
        
        if code == 0:
            # Success
            self.status_label.setText("登录成功！")
            self.status_label.setStyleSheet("color: #4caf50; font-weight: bold;")
            self.poll_timer.stop()
            
            # Save cookies
            if cookies:
                self.auth_manager.save_cookies(cookies)
                self.login_success.emit()
                # Wait a bit before closing
                await asyncio.sleep(1)
                self.accept()
                
        elif code == 86101:
            # Scanned
            # User requested to keep text fixed and avoid "false positive" updates
            # self.status_label.setText("扫描成功，请在手机上确认")
            # self.status_label.setStyleSheet("color: #ff9800;")
            pass
            
        elif code == 86038:
            # Expired
            self.status_label.setText("二维码已过期")
            self.status_label.setStyleSheet("color: #ff5555;")
            self.poll_timer.stop()
            self.refresh_btn.setVisible(True)
            
        elif code == 86090:
            # Not scanned yet, do nothing
            pass
            
        else:
            # Other error
            pass

    # Support dragging the frameless window
    def mousePressEvent(self, event):
        if event.button() == Qt.MouseButton.LeftButton:
            self.drag_pos = event.globalPosition().toPoint() - self.frameGeometry().topLeft()
            event.accept()

    def mouseMoveEvent(self, event):
        if event.buttons() == Qt.MouseButton.LeftButton:
            self.move(event.globalPosition().toPoint() - self.drag_pos)
            event.accept()



# -*- coding: utf-8 -*-
import logging
from typing import *

from .clients import ws_base
from .models import web as web_models, open_live as open_models

__all__ = (
    'HandlerInterface',
    'BaseHandler',
)

logger = logging.getLogger('blivedm')

logged_unknown_cmds = {
    'COMBO_SEND',
    'ENTRY_EFFECT',
    'HOT_RANK_CHANGED',
    'HOT_RANK_CHANGED_V2',
    'LIVE',
    'LIVE_INTERACTIVE_GAME',
    'NOTICE_MSG',
    'ONLINE_RANK_COUNT',
    'ONLINE_RANK_TOP3',
    'ONLINE_RANK_V2',
    'PK_BATTLE_END',
    'PK_BATTLE_FINAL_PROCESS',
    'PK_BATTLE_PROCESS',
    'PK_BATTLE_PROCESS_NEW',
    'PK_BATTLE_SETTLE',
    'PK_BATTLE_SETTLE_USER',
    'PK_BATTLE_SETTLE_V2',
    'PREPARING',
    'ROOM_REAL_TIME_MESSAGE_UPDATE',
    'STOP_LIVE_ROOM_LIST',
    'SUPER_CHAT_MESSAGE_JPN',
    'USER_TOAST_MSG',
    'WIDGET_BANNER',
}
"""已打日志的未知cmd"""


class HandlerInterface:
    """
    直播消息处理器接口
    """

    def handle(self, client: ws_base.WebSocketClientBase, command: dict):
        raise NotImplementedError

    def on_client_stopped(self, client: ws_base.WebSocketClientBase, exception: Optional[Exception]):
        """
        当客户端停止时调用。可以在这里close或者重新start
        """


def _make_msg_callback(method_name, message_cls):
    def callback(self: 'BaseHandler', client: ws_base.WebSocketClientBase, command: dict):
        method = getattr(self, method_name)
        return method(client, message_cls.from_command(command['data']))
    return callback


class BaseHandler(HandlerInterface):
    """
    一个简单的消息处理器实现，带消息分发和消息类型转换。继承并重写_on_xxx方法即可实现自己的处理器
    """

    def __danmu_msg_callback(self, client: ws_base.WebSocketClientBase, command: dict):
        return self._on_danmaku(client, web_models.DanmakuMessage.from_command(command['info']))

    def __danmu_msg_mirror_callback(self, client: ws_base.WebSocketClientBase, command: dict):
        message = web_models.DanmakuMessage.from_command(command['info'])
        message.is_mirror = True
        return self._on_danmaku(client, message)

    def __open_dm_mirror_callback(self, client: ws_base.WebSocketClientBase, command: dict):
        message = open_models.DanmakuMessage.from_command(command['data'])
        message.is_mirror = True
        return self._on_open_live_danmaku(client, message)

    _CMD_CALLBACK_DICT: Dict[
        str,
        Optional[Callable[
            ['BaseHandler', ws_base.WebSocketClientBase, dict],
            Any
        ]]
    ]
    """cmd -> 处理回调"""
    _CMD_CALLBACK_DICT = {
        # 收到心跳包，这是blivedm自造的消息，原本的心跳包格式不一样
        '_HEARTBEAT': _make_msg_callback('_on_heartbeat', web_models.HeartbeatMessage),
        # 弹幕
        # go-common\app\service\live\live-dm\service\v1\send.go
        'DANMU_MSG': __danmu_msg_callback,
        'DANMU_MSG_MIRROR': __danmu_msg_mirror_callback,
        # 礼物
        'SEND_GIFT': _make_msg_callback('_on_gift', web_models.GiftMessage),
        # 上舰
        'GUARD_BUY': _make_msg_callback('_on_buy_guard', web_models.GuardBuyMessage),
        # 另一个上舰消息
        'USER_TOAST_MSG_V2': _make_msg_callback('_on_user_toast_v2', web_models.UserToastV2Message),
        # 醒目留言
        'SUPER_CHAT_MESSAGE': _make_msg_callback('_on_super_chat', web_models.SuperChatMessage),
        # 删除醒目留言
        'SUPER_CHAT_MESSAGE_DELETE': _make_msg_callback('_on_super_chat_delete', web_models.SuperChatDeleteMessage),
        # 进入房间、关注主播等互动消息
        'INTERACT_WORD_V2': _make_msg_callback('_on_interact_word_v2', web_models.InteractWordV2Message),

        #
        # 开放平台消息
        #

        # 弹幕
        'LIVE_OPEN_PLATFORM_DM': _make_msg_callback('_on_open_live_danmaku', open_models.DanmakuMessage),
        'LIVE_OPEN_PLATFORM_DM_MIRROR': __open_dm_mirror_callback,
        # 礼物
        'LIVE_OPEN_PLATFORM_SEND_GIFT': _make_msg_callback('_on_open_live_gift', open_models.GiftMessage),
        # 上舰
        'LIVE_OPEN_PLATFORM_GUARD': _make_msg_callback('_on_open_live_buy_guard', open_models.GuardBuyMessage),
        # 醒目留言
        'LIVE_OPEN_PLATFORM_SUPER_CHAT': _make_msg_callback('_on_open_live_super_chat', open_models.SuperChatMessage),
        # 删除醒目留言
        'LIVE_OPEN_PLATFORM_SUPER_CHAT_DEL': _make_msg_callback(
            '_on_open_live_super_chat_delete', open_models.SuperChatDeleteMessage
        ),
        # 点赞
        'LIVE_OPEN_PLATFORM_LIKE': _make_msg_callback('_on_open_live_like', open_models.LikeMessage),
        # 进入房间
        'LIVE_OPEN_PLATFORM_LIVE_ROOM_ENTER': _make_msg_callback('_on_open_live_enter_room', open_models.RoomEnterMessage),
        # 开始直播
        'LIVE_OPEN_PLATFORM_LIVE_START': _make_msg_callback('_on_open_live_start_live', open_models.LiveStartMessage),
        # 结束直播
        'LIVE_OPEN_PLATFORM_LIVE_END': _make_msg_callback('_on_open_live_end_live', open_models.LiveEndMessage),
    }

    def handle(self, client: ws_base.WebSocketClientBase, command: dict):
        cmd = command.get('cmd', '')
        pos = cmd.find(':')  # 2019-5-29 B站弹幕升级新增了参数
        if pos != -1:
            cmd = cmd[:pos]

        if cmd not in self._CMD_CALLBACK_DICT:
            # 只有第一次遇到未知cmd时打日志
            if cmd not in logged_unknown_cmds:
                logger.warning('room=%d unknown cmd=%s, command=%s', client.room_id, cmd, command)
                logged_unknown_cmds.add(cmd)
            return

        callback = self._CMD_CALLBACK_DICT[cmd]
        if callback is not None:
            callback(self, client, command)

    def _on_heartbeat(self, client: ws_base.WebSocketClientBase, message: web_models.HeartbeatMessage):
        """收到心跳包"""

    def _on_danmaku(self, client: ws_base.WebSocketClientBase, message: web_models.DanmakuMessage):
        """弹幕"""

    def _on_gift(self, client: ws_base.WebSocketClientBase, message: web_models.GiftMessage):
        """礼物"""

    def _on_buy_guard(self, client: ws_base.WebSocketClientBase, message: web_models.GuardBuyMessage):
        """上舰"""

    def _on_user_toast_v2(self, client: ws_base.WebSocketClientBase, message: web_models.UserToastV2Message):
        """另一个上舰消息"""

    def _on_super_chat(self, client: ws_base.WebSocketClientBase, message: web_models.SuperChatMessage):
        """醒目留言"""

    def _on_super_chat_delete(self, client: ws_base.WebSocketClientBase, message: web_models.SuperChatDeleteMessage):
        """删除醒目留言"""

    def _on_interact_word_v2(self, client: ws_base.WebSocketClientBase, message: web_models.InteractWordV2Message):
        """进入房间、关注主播等互动消息"""

    #
    # 开放平台消息
    #

    def _on_open_live_danmaku(self, client: ws_base.WebSocketClientBase, message: open_models.DanmakuMessage):
        """弹幕"""

    def _on_open_live_gift(self, client: ws_base.WebSocketClientBase, message: open_models.GiftMessage):
        """礼物"""

    def _on_open_live_buy_guard(self, client: ws_base.WebSocketClientBase, message: open_models.GuardBuyMessage):
        """上舰"""

    def _on_open_live_super_chat(self, client: ws_base.WebSocketClientBase, message: open_models.SuperChatMessage):
        """醒目留言"""

    def _on_open_live_super_chat_delete(
        self, client: ws_base.WebSocketClientBase, message: open_models.SuperChatDeleteMessage
    ):
        """删除醒目留言"""

    def _on_open_live_like(self, client: ws_base.WebSocketClientBase, message: open_models.LikeMessage):
        """点赞"""

    def _on_open_live_enter_room(self, client: ws_base.WebSocketClientBase, message: open_models.RoomEnterMessage):
        """进入房间"""

    def _on_open_live_start_live(self, client: ws_base.WebSocketClientBase, message: open_models.LiveStartMessage):
        """开始直播"""

    def _on_open_live_end_live(self, client: ws_base.WebSocketClientBase, message: open_models.LiveEndMessage):
        """结束直播"""



# -*- coding: utf-8 -*-
import asyncio
import aiohttp
import qrcode
import keyring
import json
import logging
import time
from typing import Optional, Dict, Tuple, Any
from io import BytesIO
from PIL import Image

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

SERVICE_ID = "bilihud"
USERNAME_KEY = "bilibili_cookies"

class AuthManager:
    """
    Manage Bilibili Authentication via QR Code
    """
    
    BASE_URL = "https://passport.bilibili.com/x/passport-login/web/qrcode/generate"
    POLL_URL = "https://passport.bilibili.com/x/passport-login/web/qrcode/poll"

    def __init__(self):
        self.session: Optional[aiohttp.ClientSession] = None

    async def get_qrcode(self) -> Tuple[Optional[str], Optional[str]]:
        """
        Get QR code URL and Key
        Returns: (url, qrcode_key)
        """
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36'
        }
        async with aiohttp.ClientSession(headers=headers) as session:
            try:
                async with session.get(self.BASE_URL) as response:
                    data = await response.json()
                    if data['code'] == 0:
                        return data['data']['url'], data['data']['qrcode_key']
                    else:
                        logger.error(f"Failed to get QR code: {data}")
                        return None, None
            except Exception as e:
                logger.error(f"Exception requesting QR code: {e}")
                return None, None

    def generate_qr_image(self, url: str) -> Optional[BytesIO]:
        """
        Generate QR code image from URL
        Returns: BytesIO object containing PNG image
        """
        try:
            qr = qrcode.QRCode(
                version=1,
                error_correction=qrcode.constants.ERROR_CORRECT_L,
                box_size=10,
                border=4,
            )
            qr.add_data(url)
            qr.make(fit=True)

            img = qr.make_image(fill_color="black", back_color="white")
            bio = BytesIO()
            img.save(bio, format="PNG")
            bio.seek(0)
            return bio
        except Exception as e:
            logger.error(f"Failed to generate QR image: {e}")
            return None

    async def poll_status(self, qrcode_key: str) -> Tuple[int, str, Optional[Dict]]:
        """
        Poll login status
        Returns: (code, message, cookies_dict)
        code: 0=Success, 86101=Scanned, 86090=Not Scanned, 86038=Expired
        """
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36'
        }
        async with aiohttp.ClientSession(headers=headers) as session:
            try:
                params = {'qrcode_key': qrcode_key}
                async with session.get(self.POLL_URL, params=params) as response:
                    data = await response.json()
                    
                    if data['code'] == 0:
                        code = data['data']['code']
                        msg = data['data']['message']
                        
                        if code == 0:
                            # Login success, extract cookies
                            cookies = {}
                            for cookie in session.cookie_jar:
                                cookies[cookie.key] = cookie.value
                            
                            # Also check Set-Cookie headers directly if needed, usually cookie_jar has them
                            # Filter for essential cookies
                            essential_keys = ['SESSDATA', 'bili_jct', 'DedeUserID', 'DedeUserID__ckMd5']
                            filtered_cookies = {k: v for k, v in cookies.items() if k in essential_keys}
                            
                            return 0, "登录成功", filtered_cookies
                        
                        return code, msg, None
                    else:
                        return -1, f"API Error: {data['code']}", None
            except Exception as e:
                logger.error(f"Exception polling status: {e}")
                return -1, str(e), None

    def save_cookies(self, cookies: Dict[str, str]) -> bool:
        """
        Save cookies securely using keyring
        """
        try:
            cookie_json = json.dumps(cookies)
            keyring.set_password(SERVICE_ID, USERNAME_KEY, cookie_json)
            return True
        except Exception as e:
            logger.error(f"Failed to save cookies to keyring: {e}")
            return False

    def load_cookies(self) -> Optional[Dict[str, str]]:
        """
        Load cookies from keyring
        """
        try:
            cookie_json = keyring.get_password(SERVICE_ID, USERNAME_KEY)
            if cookie_json:
                return json.loads(cookie_json)
            return None
        except Exception as e:
            logger.error(f"Failed to load cookies from keyring: {e}")
            return None
            
    def clear_cookies(self):
        """Clear stored cookies"""
        try:
            keyring.delete_password(SERVICE_ID, USERNAME_KEY)
        except Exception as e:
            logger.error(f"Failed to delete cookies: {e}")

    async def validate_session(self, cookies: Dict[str, str]) -> bool:
        """
        Validate if the current cookies are logged in
        """
        url = "https://api.bilibili.com/x/web-interface/nav"
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36'
        }
        try:
            async with aiohttp.ClientSession(cookies=cookies, headers=headers) as session:
                async with session.get(url) as resp:
                    data = await resp.json()
                    if data['code'] == 0:
                         return data['data'].get('isLogin', False)
                    return False
        except Exception as e:
            logger.error(f"Session validation failed: {e}")
            return False




from bilihud.utils import validate_room_id

def test_validate_room_id():
    """Test room ID validation logic"""
    assert validate_room_id("123") is True
    assert validate_room_id("2145") is True
    
    # Invalid cases
    assert validate_room_id("0") is False
    assert validate_room_id("-1") is False
    assert validate_room_id("abc") is False
    assert validate_room_id("") is False



# -*- coding: utf-8 -*-
import sys
import os
import asyncio
import qasync
import ctypes
from ctypes import c_void_p, c_int, c_ulong
from typing import Optional

from PyQt6.QtWidgets import (
    QApplication, QWidget, QVBoxLayout, QHBoxLayout, QLabel,
    QListWidget, QListWidgetItem, QLineEdit, QPushButton, QFrame,
    QGraphicsDropShadowEffect, QSystemTrayIcon, QMenu,
    QDialog, QSizePolicy, QAbstractItemView, QSizeGrip
)
from PyQt6.QtGui import (
    QCloseEvent, QFont, QColor, QPalette, QIcon, QCursor, 
    QLinearGradient, QBrush, QPainter, QAction
)
from PyQt6.QtCore import (
    QTimer, Qt, pyqtSignal, QSize, QPoint, QRect
)

import blivedm.models.web as web_models
from .danmaku_client import DanmakuClient
from .utils import load_config, save_config
from .qr_login_dialog import QRLoginDialog
from .auth import AuthManager

class ModernInputWidget(QWidget):
    """
    一个现代化的输入框组件，包含圆形输入框和发送按钮
    """
    send_requested = pyqtSignal(str)

    def __init__(self, parent=None, placeholder="发送弹幕..."):
        super().__init__(parent)
        self.layout = QHBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.setSpacing(6)

        # 输入框
        self.input_edit = QLineEdit()
        self.input_edit.setPlaceholderText(placeholder)
        self.input_edit.setStyleSheet("""
            QLineEdit {
                background-color: rgba(255, 255, 255, 30);
                color: white;
                border: 1px solid rgba(255, 255, 255, 50);
                border-radius: 13px;
                padding: 4px 10px;
                font-family: 'Segoe UI', 'Microsoft YaHei';
                font-size: 12px;
                selection-background-color: rgba(255, 255, 255, 100);
            }
            QLineEdit:focus {
                background-color: rgba(255, 255, 255, 50);
                border: 1px solid rgba(255, 255, 255, 150);
            }
        """)
        self.input_edit.returnPressed.connect(self.on_send)
        
        # 发送按钮
        self.send_btn = QPushButton("发送")
        self.send_btn.setCursor(Qt.CursorShape.PointingHandCursor)
        self.send_btn.setFixedSize(46, 26)
        self.send_btn.setStyleSheet("""
            QPushButton {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0, stop:0 #4FacFe, stop:1 #00f2fe);
                color: white;
                border: none;
                border-radius: 13px;
                font-weight: bold;
                font-size: 11px;
                font-family: 'Segoe UI', 'Microsoft YaHei';
            }
            QPushButton:hover {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0, stop:0 #66b5ff, stop:1 #33f5ff);
            }
            QPushButton:pressed {
                background: #00bcd4;
            }
        """)
        self.send_btn.clicked.connect(self.on_send)

        self.layout.addWidget(self.input_edit)
        self.layout.addWidget(self.send_btn)

    def on_send(self):
        text = self.input_edit.text().strip()
        if text:
            self.send_requested.emit(text)
            self.input_edit.clear()

    def setFocus(self):
        self.input_edit.setFocus()


class DanmakuInputDialog(QDialog):
    """全局弹幕输入框 (用于游戏模式/快捷唤起)"""
    
    send_message = pyqtSignal(str)
    
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowFlags(Qt.WindowType.FramelessWindowHint | Qt.WindowType.WindowStaysOnTopHint | Qt.WindowType.Window)
        self.setAttribute(Qt.WidgetAttribute.WA_InputMethodEnabled)
        self.setAttribute(Qt.WidgetAttribute.WA_TranslucentBackground)
        self.resize(450, 60)
        
        # 整体布局
        layout = QHBoxLayout(self)
        layout.setContentsMargins(5, 5, 5, 5)
        
        # 背景容器 (实现Glass效果)
        self.container = QFrame(self)
        self.container.setStyleSheet("""
            QFrame {
                background-color: rgba(20, 20, 30, 220);
                border-radius: 15px;
                border: 1px solid rgba(255, 255, 255, 30);
            }
        """)
        
        # 加阴影
        shadow = QGraphicsDropShadowEffect(self)
        shadow.setBlurRadius(15)
        shadow.setOffset(0, 3)
        shadow.setColor(QColor(0, 0, 0, 150))
        self.container.setGraphicsEffect(shadow)
        
        container_layout = QHBoxLayout(self.container)
        container_layout.setContentsMargins(10, 8, 10, 8)
        
        self.input_widget = ModernInputWidget(self, placeholder="输入弹幕... [ESC关闭]")
        self.input_widget.send_requested.connect(self.on_send)
        
        container_layout.addWidget(self.input_widget)
        layout.addWidget(self.container)
        
    def on_send(self, text):
        self.send_message.emit(text)
        self.hide() # 发送后隐藏
            
    def showEvent(self, event):
        super().showEvent(event)
        self.input_widget.setFocus()
        # 居中显示在屏幕下方
        screen = QApplication.primaryScreen().geometry()
        self.move(
            screen.width() // 2 - self.width() // 2,
            int(screen.height() * 0.8)
        )
        self.activateWindow()
        self.raise_()
        
    def keyPressEvent(self, event):
        if event.key() == Qt.Key.Key_Escape:
            self.hide()
        super().keyPressEvent(event)

class X11Helper:
    """X11辅助类，用于直接调用XShape扩展实现点击穿透"""
    _x11 = None
    _xext = None
    
    @classmethod
    def init(cls):
        if cls._x11: return
        try:
            cls._x11 = ctypes.cdll.LoadLibrary('libX11.so.6')
            cls._xext = ctypes.cdll.LoadLibrary('libXext.so.6')
            
            cls._x11.XOpenDisplay.restype = c_void_p
            cls._x11.XOpenDisplay.argtypes = [c_void_p]
            cls._x11.XFlush.argtypes = [c_void_p]
            cls._x11.XCloseDisplay.argtypes = [c_void_p]
            
            # XShapeCombineRectangles(display, dest, dest_kind, x_off, y_off, rectangles, n_rects, op, ordering)
            cls._xext.XShapeCombineRectangles.argtypes = [
                c_void_p, c_ulong, c_int, c_int, c_int, c_void_p, c_int, c_int, c_int
            ]
            
            # XShapeCombineMask(display, dest, dest_kind, x_off, y_off, src, op)
            cls._xext.XShapeCombineMask.argtypes = [
                c_void_p, c_ulong, c_int, c_int, c_int, c_void_p, c_int
            ]
        except Exception as e:
            print(f"X11 init failed: {e}")

    @classmethod
    def set_click_through(cls, win_id, enabled):
        """设置窗口是否通过Input Shape完全穿透"""
        if sys.platform != 'linux': return
        
        cls.init()
        if not cls._x11 or not cls._xext: return
        
        display = cls._x11.XOpenDisplay(None)
        if not display:
            print("Failed to open X Display")
            return
        
        ShapeInput = 2
        ShapeSet = 0
        
        try:
            if enabled:
                # 设置输入形状为空（0个矩形），使窗口对输入事件完全透明
                cls._xext.XShapeCombineRectangles(
                    display, win_id, ShapeInput, 0, 0, None, 0, ShapeSet, 0
                )
            else:
                # 恢复默认输入形状（重置为None Mask），允许接收输入
                cls._xext.XShapeCombineMask(
                    display, win_id, ShapeInput, 0, 0, None, ShapeSet
                )
            cls._x11.XFlush(display)
        finally:
            cls._x11.XCloseDisplay(display)


class DanmakuItemWidget(QFrame):
    """单个弹幕项的自定义控件"""

    def __init__(self, message: web_models.DanmakuMessage | web_models.GiftMessage | web_models.InteractWordV2Message, parent=None):
        super().__init__(parent)
        
        if isinstance(message, web_models.DanmakuMessage):
            self.setup_danmaku_ui(message)
        elif isinstance(message, web_models.GiftMessage):
            self.setup_gift_ui(message)
        elif isinstance(message, web_models.InteractWordV2Message):
            self.setup_interact_ui(message)

    def setup_danmaku_ui(self, danmaku_msg: web_models.DanmakuMessage):
        # 设置框架样式 - 透明背景
        self.setStyleSheet("background-color: transparent;")

        # 主布局
        main_layout = QHBoxLayout()
        main_layout.setContentsMargins(5, 2, 5, 2)
        main_layout.setSpacing(4)

        # 文字阴影效果
        def create_shadow_effect(color=QColor(0, 0, 0, 200)):
            shadow = QGraphicsDropShadowEffect()
            shadow.setBlurRadius(2)
            shadow.setOffset(1, 1)
            shadow.setColor(color)
            return shadow

        # 用户名标签
        username_label = QLabel(danmaku_msg.uname)
        username_label.setGraphicsEffect(create_shadow_effect())
        username_label.setStyleSheet(f"""
            QLabel {{
                color: {self.get_user_color(danmaku_msg)};
                font-weight: bold;
                font-size: 12px;
                font-family: 'Segoe UI', 'Microsoft YaHei', sans-serif;
            }}
        """)

        # 冒号标签
        colon_label = QLabel(":")
        colon_label.setGraphicsEffect(create_shadow_effect())
        colon_label.setStyleSheet("""
            QLabel {
                color: white;
                font-size: 12px;
                font-family: 'Segoe UI', 'Microsoft YaHei', sans-serif;
            }
        """)

        # 弹幕内容标签
        content_label = QLabel(danmaku_msg.msg)
        content_label.setGraphicsEffect(create_shadow_effect())
        content_label.setStyleSheet("""
            QLabel {
                color: white;
                font-size: 13px;
                font-weight: 500;
                font-family: 'Segoe UI', 'Microsoft YaHei', sans-serif;
            }
        """)
        content_label.setWordWrap(True)

        # 添加控件到布局
        main_layout.addWidget(username_label)
        main_layout.addWidget(colon_label)
        main_layout.addWidget(content_label, 1) # content stretches

        self.setLayout(main_layout)

    def get_user_color(self, danmaku_msg: web_models.DanmakuMessage) -> str:
        """根据用户等级获取用户名颜色"""
        if getattr(danmaku_msg, 'is_system_error', False):
            return "#FF5555"  # 红色 (系统错误)
        elif getattr(danmaku_msg, 'is_system_info', False):
            return "#AAAAAA"  # 灰色 (系统信息)
            
        if danmaku_msg.privilege_type > 0:
            return "#FFD700"  # 金色 (舰队)
        elif danmaku_msg.vip or danmaku_msg.svip:
            return "#FF69B4"  # 粉色 (VIP)
        elif danmaku_msg.admin:
            return "#FF4500"  # 红橙色 (房管)
        return "#66CCFF"  # 天蓝色 (普通)

    def setup_gift_ui(self, gift_msg: web_models.GiftMessage):
        """设置礼物消息样式"""
        self.setStyleSheet("background-color: transparent;")
        main_layout = QHBoxLayout(self)
        main_layout.setContentsMargins(5, 2, 5, 2)
        main_layout.setSpacing(4)
        
        shadow = QGraphicsDropShadowEffect()
        shadow.setBlurRadius(2); shadow.setOffset(1, 1); shadow.setColor(QColor(0,0,0,200))

        # 用户名
        user_label = QLabel(gift_msg.uname)
        user_label.setGraphicsEffect(shadow)
        user_label.setStyleSheet("color: #FFD700; font-weight: bold; font-size: 12px; font-family: 'Microsoft YaHei';")
        
        # 动作
        action_label = QLabel(f" {gift_msg.action} ")
        action_label.setStyleSheet("color: #FF66CC; font-size: 12px;")
        
        # 礼物名
        gift_label = QLabel(f"{gift_msg.gift_name} x{gift_msg.num}")
        gift_label.setStyleSheet("color: #FF66CC; font-weight: bold; font-size: 12px;")
        
        main_layout.addWidget(user_label)
        main_layout.addWidget(action_label)
        main_layout.addWidget(gift_label)
        main_layout.addStretch()

    def setup_interact_ui(self, interact_msg: web_models.InteractWordV2Message):
        """设置互动消息样式 (进房/关注)"""
        self.setStyleSheet("background-color: transparent;")
        main_layout = QHBoxLayout(self)
        main_layout.setContentsMargins(5, 2, 5, 2)
        main_layout.setSpacing(4)
        
        user_label = QLabel(interact_msg.username)
        user_label.setStyleSheet("color: #AAAAAA; font-size: 11px; font-weight: bold;")
        
        msg_type_map = {1: '进入直播间', 2: '关注了主播', 3: '分享了直播间'}
        action_text = msg_type_map.get(interact_msg.msg_type, '进入直播间')
        
        info_label = QLabel(f" {action_text}")
        info_label.setStyleSheet("color: #AAAAAA; font-size: 11px;")
        
        main_layout.addWidget(user_label)
        main_layout.addWidget(info_label)
        main_layout.addStretch()


class DanmakuWidget(QWidget):
    """弹幕显示窗口"""

    danmaku_received = pyqtSignal(web_models.DanmakuMessage)
    gift_received = pyqtSignal(web_models.GiftMessage)
    interact_received = pyqtSignal(web_models.InteractWordV2Message)

    def __init__(self, room_id: int = 0, sessdata: str = ''):
        super().__init__()
        self.room_id = room_id
        self.sessdata = sessdata
        self.danmaku_client: Optional[DanmakuClient] = None
        self.is_gaming_mode = False
        
        self.setup_window_properties()
        self.init_ui()
        self.setup_tray_icon()
        self.setup_danmaku_client()
        
        # 加载保存的配置
        config = load_config()
        if 'room_id' in config:
            self.room_id = config['room_id']
        
        # 初始化房间号
        self.room_id_input.setText(str(self.room_id))

    def setup_window_properties(self):
        """设置基本的窗口属性"""
        self.resize(300, 450)
        # 居中屏幕
        screen_geo = QApplication.primaryScreen().geometry()
        self.move(
            screen_geo.width() - 330, # 默认靠右一些
            100
        )
        self.setWindowTitle("Danmaku Overlay")
        
        # 基础无边框和置顶设置
        # REMOVED: Qt.X11BypassWindowManagerHint to fix input focus
        flags = (
            Qt.WindowType.FramelessWindowHint | 
            Qt.WindowType.WindowStaysOnTopHint | 
            Qt.WindowType.Window # Using Window instead of Tool
        )
            
        self.setWindowFlags(flags)
        self.setAttribute(Qt.WidgetAttribute.WA_TranslucentBackground)

    def paintEvent(self, event):
        """自定义绘制背景，实现轻微的渐变面板效果 (非穿透模式下)"""
        if not self.is_gaming_mode:
            painter = QPainter(self)
            painter.setRenderHint(QPainter.RenderHint.Antialiasing)
            
            # 使用半透明黑色背景
            painter.setBrush(QBrush(QColor(0, 0, 0, 120)))
            painter.setPen(Qt.PenStyle.NoPen)
            painter.drawRoundedRect(self.rect(), 8, 8)
            super().paintEvent(event)

    def init_ui(self):
        """初始化UI界面"""
        # 主布局
        self.main_layout = QVBoxLayout()
        self.main_layout.setContentsMargins(10, 10, 10, 10)
        self.main_layout.setSpacing(8)

        # --- 控制栏 (Header) ---
        self.header_widget = QWidget()
        self.header_layout = QHBoxLayout(self.header_widget)
        self.header_layout.setContentsMargins(0, 0, 0, 0)
        self.header_layout.setSpacing(8)

        # 标题
        self.title_label = QLabel("BILIHUD")
        self.title_label.setStyleSheet("""
            color: rgba(255, 255, 255, 200); 
            font-weight: 900; 
            font-family: 'Arial Black';
            font-size: 12px;
            letter-spacing: 0.5px;
        """)
        
        # 房间号输入
        self.room_id_input = QLineEdit(str(self.room_id))
        self.room_id_input.setPlaceholderText("ID")
        self.room_id_input.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.room_id_input.setStyleSheet("""
            QLineEdit {
                border: 1px solid rgba(255, 255, 255, 30);
                border-radius: 4px;
                padding: 2px 4px;
                background: rgba(0, 0, 0, 50);
                color: #ddd;
                font-weight: bold;
                max-width: 70px;
                font-size: 11px;
            }
            QLineEdit:focus {
                border-color: rgba(255, 255, 255, 100);
            }
        """)
        self.room_id_input.editingFinished.connect(self.save_room_id)

        # 按钮样式
        btn_style = """
            QPushButton {
                color: white;
                background-color: rgba(255, 255, 255, 20);
                border: 1px solid rgba(255, 255, 255, 30);
                border-radius: 4px;
                padding: 2px 8px;
                font-size: 11px;
            }
            QPushButton:hover {
                background-color: rgba(255, 255, 255, 40);
            }
            QPushButton:checked {
                background-color: rgba(76, 175, 80, 150);
                border-color: rgba(76, 175, 80, 200);
            }
        """

        # 连接按钮
        self.connect_button = QPushButton("连接")
        self.connect_button.setCursor(Qt.CursorShape.PointingHandCursor)
        self.connect_button.setCheckable(True)
        self.connect_button.setStyleSheet(btn_style)
        self.connect_button.clicked.connect(self.toggle_connection)
        
        # 游戏模式切换按钮
        self.gaming_mode_btn = QPushButton("锁定穿透")
        self.gaming_mode_btn.setCursor(Qt.CursorShape.PointingHandCursor)
        self.gaming_mode_btn.setCheckable(True)
        self.gaming_mode_btn.setStyleSheet(btn_style)
        self.gaming_mode_btn.clicked.connect(self.toggle_gaming_mode)

        # 关闭按钮 (右上角小圆点)
        close_btn = QPushButton("×")
        close_btn.setCursor(Qt.CursorShape.PointingHandCursor)
        close_btn.setFixedSize(20, 20)
        close_btn.setStyleSheet("""
            QPushButton {
                color: rgba(255,255,255,150);
                background: transparent;
                border: 1px solid rgba(255,0,0,50);
                border-radius: 10px;
                font-weight: bold;
                font-size: 14px;
                padding-bottom: 2px;
            }
            QPushButton:hover {
                background: rgba(255, 0, 0, 180);
                color: white;
                border-color: transparent;
            }
        """)
        close_btn.clicked.connect(self.close)

        # 组装 Header
        self.header_layout.addWidget(self.title_label)
        self.header_layout.addWidget(self.room_id_input)
        self.header_layout.addWidget(self.connect_button)
        self.header_layout.addWidget(self.gaming_mode_btn)
        self.header_layout.addStretch()
        self.header_layout.addWidget(close_btn)

        # --- 弹幕列表 ---
        self.danmaku_list = QListWidget()
        self.danmaku_list.setStyleSheet("background: transparent; border: none;")
        self.danmaku_list.setVerticalScrollMode(QAbstractItemView.ScrollMode.ScrollPerPixel)
        self.danmaku_list.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.danmaku_list.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        self.danmaku_list.setSelectionMode(QAbstractItemView.SelectionMode.NoSelection)
        self.danmaku_list.setFocusPolicy(Qt.FocusPolicy.NoFocus)

        # 滚动条样式美化
        self.danmaku_list.verticalScrollBar().setStyleSheet("""
            QScrollBar:vertical {
                border: none;
                background: rgba(0, 0, 0, 0);
                width: 4px;
                margin: 0;
            }
            QScrollBar::handle:vertical {
                background: rgba(255, 255, 255, 50);
                min-height: 20px;
                border-radius: 2px;
            }
            QScrollBar::add-line:vertical, QScrollBar::sub-line:vertical { height: 0px; }
        """)

        # --- 底部输入区域 (新) ---
        self.input_area = ModernInputWidget(self)
        self.input_area.send_requested.connect(self.trigger_send)

        # 组装 Main
        self.main_layout.addWidget(self.header_widget)
        self.main_layout.addWidget(self.danmaku_list)
        self.main_layout.addWidget(self.input_area) # 放在底部
        
        self.setLayout(self.main_layout)
        
        # 信号连接
        self.danmaku_received.connect(self.add_message)
        self.gift_received.connect(self.add_message)
        self.interact_received.connect(self.add_message)
        
        # 初始化全局输入框
        self.input_dialog = DanmakuInputDialog(None)
        self.input_dialog.send_message.connect(self.trigger_send)

        # 拖拽移动相关变量
        self._dragging = False
        self._drag_position = QPoint()
        
        # 大小调整手柄
        self.size_grip = QSizeGrip(self)
        self.size_grip.setStyleSheet("""
            QSizeGrip {
                background-color: transparent;
                width: 16px; 
                height: 16px;
            }
        """)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        
        # 1. 更新SizeGrip位置
        rect = self.rect()
        self.size_grip.move(
            rect.right() - self.size_grip.width(),
            rect.bottom() - self.size_grip.height()
        )
        
        # 2. 在非穿透模式下，根据宽度动态调整列表项高度以支持换行
        if not self.is_gaming_mode:
            self.adjust_list_items_height()

    def adjust_list_items_height(self):
        """重新计算所有列表项的高度"""
        # 获取列表视口宽度（减去滚动条可能的宽度，虽然我们隐藏了滚动条）
        width = self.danmaku_list.viewport().width()
        if width <= 0: return

        count = self.danmaku_list.count()
        for i in range(count):
            item = self.danmaku_list.item(i)
            widget = self.danmaku_list.itemWidget(item)
            if widget:
                # 临时固定宽度以计算正确的高度
                widget.setFixedWidth(width)
                size_hint = widget.sizeHint()
                # 恢复宽度限制
                widget.setMinimumWidth(0)
                widget.setMaximumWidth(16777215) 
                
                # 如果高度变化了，更新Item
                if item.sizeHint().height() != size_hint.height():
                    item.setSizeHint(size_hint)


    def setup_tray_icon(self):
        """初始化系统托盘图标"""
        self.tray_icon = QSystemTrayIcon(self)
        
        # 加载图标
        icon_path = os.path.join(os.path.dirname(__file__), 'assets', 'icon.png')
        if os.path.exists(icon_path):
            icon = QIcon(icon_path)
            self.tray_icon.setIcon(icon)
            self.setWindowIcon(icon)
        else:
            print(f"Icon not found at {icon_path}")
        
        # 创建托盘菜单
        tray_menu = QMenu()
        tray_menu.setStyleSheet("""
            QMenu {
                background-color: #2b2b2b;
                color: #ffffff;
                border: 1px solid #3d3d3d;
            }
            QMenu::item {
                padding: 5px 20px;
            }
            QMenu::item:selected {
                background-color: #3d3d3d;
            }
        """)
        
        self.tray_send_action = QAction("发送弹幕", self)
        self.tray_send_action.triggered.connect(self.open_input_dialog)
        tray_menu.addAction(self.tray_send_action)
        
        tray_menu.addSeparator()
        
        self.tray_toggle_action = QAction("显示/隐藏", self)
        self.tray_toggle_action.triggered.connect(self.toggle_visibility)
        tray_menu.addAction(self.tray_toggle_action)
        
        self.tray_gaming_action = QAction("锁定穿透 (游戏模式)", self)
        self.tray_gaming_action.setCheckable(True)
        self.tray_gaming_action.triggered.connect(self.toggle_gaming_mode_from_tray)
        tray_menu.addAction(self.tray_gaming_action)
        
        tray_menu.addSeparator()

        self.tray_login_action = QAction("扫码登录", self)
        self.tray_login_action.triggered.connect(self.open_qr_login)
        tray_menu.addAction(self.tray_login_action)
        
        quit_action = QAction("退出程序", self)
        quit_action.triggered.connect(self.quit_app)
        tray_menu.addAction(quit_action)
        
        self.tray_icon.setContextMenu(tray_menu)
        self.tray_icon.show()
        
        self.tray_icon.activated.connect(self.on_tray_activated)

    def add_system_message(self, message: str, level: str = "info"):
        """添加系统消息到列表"""
        class SystemMessage:
            def __init__(self, msg, level):
                self.uname = " [系统]"
                self.msg = msg
                self.privilege_type = 0
                self.vip = False
                self.svip = False
                self.admin = False
                self.is_system_error = (level == "error")
                self.is_system_info = (level == "info")
        
        msg_obj = SystemMessage(message, level)
        self.add_message(msg_obj)

    async def _send_danmaku_task(self, text: str):
        """实际执行发送弹幕的Task"""
        if self.danmaku_client:
            success, msg = await self.danmaku_client.send_danmaku(text)
            if success:
                # print(f"弹幕发送成功: {text}")
                # 可选：发送成功也显示一条本地回显，或者直接等服务器下发
                pass 
            else:
                self.add_system_message(f"发送失败: {msg}", "error")
                print(f"弹幕发送失败: {msg}")
        else:
             self.add_system_message("未连接直播间，无法发送", "error")
             print("未连接，无法发送")

    def trigger_send(self, text: str):
        """处理发送弹幕请求"""
        if not text: return
        asyncio.create_task(self._send_danmaku_task(text))

    def open_input_dialog(self):
        """打开全局输入框"""
        self.input_dialog.show()
        self.input_dialog.activateWindow()

    def on_tray_activated(self, reason):
        if reason == QSystemTrayIcon.ActivationReason.DoubleClick:
            self.toggle_visibility()

    def toggle_visibility(self):
        if self.isVisible():
            self.hide()
        else:
            self.show()
            self.activateWindow()

    def quit_app(self):
        QApplication.quit()

    def toggle_gaming_mode_from_tray(self, checked):
        """从托盘切换游戏模式"""
        # 避免递归更新
        if self.is_gaming_mode != checked:
            self.set_gaming_mode(checked)

    def toggle_gaming_mode(self):
        """切换鼠标穿透/游戏模式"""
        new_state = not self.is_gaming_mode
        self.set_gaming_mode(new_state)

    def set_gaming_mode(self, enabled: bool):
        self.is_gaming_mode = enabled
        
        # 保存当前位置和大小
        current_geo = self.geometry()
        
        # 同步各按钮状态
        self.tray_gaming_action.setChecked(enabled)
        self.gaming_mode_btn.setChecked(enabled)
        
        # [Critical Fix] 重新构建Flags
        flags = Qt.WindowType.FramelessWindowHint | Qt.WindowType.WindowStaysOnTopHint | Qt.WindowType.Window

        if enabled:
            # --- 开启穿透模式 (Gaming Mode) ---
            
            # 1. 穿透模式核心Flags
            # X11BypassWindowManagerHint: 绕过WM，确保能在全屏游戏之上显示
            flags |= Qt.WindowType.X11BypassWindowManagerHint
            # WindowTransparentForInput: 输入事件穿透 (配合XShape)
            flags |= Qt.WindowType.WindowTransparentForInput
            # WindowDoesNotAcceptFocus: 拒绝焦点，防止抢占游戏输入
            flags |= Qt.WindowType.WindowDoesNotAcceptFocus

            # 2. UI调整
            self.header_widget.hide()
            self.input_area.hide()
            self.danmaku_list.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
            
            self.danmaku_list.setStyleSheet("""
                QListWidget {
                    background: transparent;
                    border: 2px dashed rgba(255, 255, 255, 30);
                    border-radius: 8px;
                }
            """)
            
            # 3. 属性设置
            self.setAttribute(Qt.WidgetAttribute.WA_TransparentForMouseEvents, True)
            self.setAttribute(Qt.WidgetAttribute.WA_ShowWithoutActivating, True) # 防止show()时触发activate导致日志警告
            
            self.tray_icon.showMessage(
                "Danmaku Overlay", 
                "已进入穿透模式 (游戏覆盖)\n弹幕将显示在最顶层，鼠标操作将穿透。", 
                QSystemTrayIcon.MessageIcon.Information, 
                2000
            )
        else:
            # --- 关闭穿透模式 (Normal Mode) ---
            
            # 1. Normal Mode Flags
            # 不需要 X11Bypass，也不需要 TransparentForInput
            # 只是普通置顶无边框窗口
            
            # 2. UI调整
            self.header_widget.show()
            self.input_area.show()
            self.danmaku_list.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
            self.danmaku_list.setStyleSheet("background: transparent; border: none;")

            self.setAttribute(Qt.WidgetAttribute.WA_TransparentForMouseEvents, False)
            self.setAttribute(Qt.WidgetAttribute.WA_ShowWithoutActivating, False)
        
        # 1. 先进行销毁操作 (Hide & Set Flags)
        self.hide()
        self.setWindowFlags(flags)
        
        # 2. [Defer Fix] 延迟执行显示操作
        # 切换 BypassWindowManagerHint 会导致 Native Window 销毁重建
        # 如果同步执行 show()，可能导致 X11 状态未同步而无法映射窗口
        def restore_window_state():
            # 恢复位置 (在窗口重建后应用)
            self.setGeometry(current_geo)
            
            # 显示并置顶
            self.show()
            self.raise_()
            
            if not enabled:
                self.activateWindow()
                
            # XShape 穿透
            try:
                wid = int(self.winId())
                if wid > 0:
                    X11Helper.set_click_through(wid, enabled)
            except Exception as e:
                print(f"Failed to set X11 settings: {e}")

        # 使用 50ms 延时确保事件循环处理完窗口销毁
        QTimer.singleShot(50, restore_window_state)

    # --- 鼠标拖拽移动窗口逻辑 ---
    def mousePressEvent(self, event):
        if not self.is_gaming_mode and event.button() == Qt.MouseButton.LeftButton:
            self._dragging = True
            self._drag_position = event.globalPosition().toPoint() - self.frameGeometry().topLeft()
            event.accept()

    def mouseMoveEvent(self, event):
        if self._dragging:
            self.move(event.globalPosition().toPoint() - self._drag_position)
            event.accept()

    def mouseReleaseEvent(self, event):
        self._dragging = False

    # --- 客户端逻辑 ---
    
    def setup_danmaku_client(self):
        self.danmaku_client = None

    @qasync.asyncSlot()
    async def toggle_connection(self):
        """切换连接状态"""
        if self.danmaku_client is None or not self.danmaku_client.client:
            # 连接
            try:
                self.room_id = int(self.room_id_input.text())
                self.room_id_input.setText(str(self.room_id))
                
                self.danmaku_client = DanmakuClient(self.room_id, self.sessdata)
                self.danmaku_client.set_danmaku_callback(self.on_danmaku_received)
                self.danmaku_client.set_gift_callback(self.on_gift_received)
                self.danmaku_client.set_interact_callback(self.on_interact_received)
                self.danmaku_client.set_login_failed_callback(self.on_login_failed)
                
                # 保存房间号
                save_config({'room_id': self.room_id})
                
                # Update UI to connecting state
                self.connect_button.setText("连接中...")
                self.connect_button.setEnabled(False)
                
                await self.danmaku_client.start()
                
                self.connect_button.setText("断开")
                self.connect_button.setChecked(True)
                self.connect_button.setEnabled(True)
                self.connect_button.setStyleSheet("""
                    QPushButton {
                        background-color: rgba(244, 67, 54, 150);
                        color: white; 
                        border: 1px solid rgba(244, 67, 54, 200);
                        border-radius: 6px; padding: 4px 10px;
                    }
                    QPushButton:hover { background-color: rgba(244, 67, 54, 200); }
                """)
            except Exception as e:
                self.connect_button.setText("连接")
                self.connect_button.setEnabled(True)
                print(f"Connection failed: {e}")
        else:
            # 断开
            if self.danmaku_client is not None:
                await self.danmaku_client.stop()
            self.danmaku_client = None
            self.connect_button.setText("连接")
            self.connect_button.setChecked(False)
            self.connect_button.setStyleSheet("""
                QPushButton {
                    color: white;
                    background-color: rgba(255, 255, 255, 20);
                    border: 1px solid rgba(255, 255, 255, 30);
                    border-radius: 6px;
                    padding: 4px 10px;
                }
                QPushButton:hover { background-color: rgba(255, 255, 255, 40); }
                QPushButton:checked { background-color: rgba(76, 175, 80, 150); }
            """)
            
    def save_room_id(self):
        try:
            self.room_id = int(self.room_id_input.text())
        except ValueError:
            self.room_id_input.setText(str(self.room_id))

    def on_danmaku_received(self, danmaku_msg: web_models.DanmakuMessage):
        self.danmaku_received.emit(danmaku_msg)

    def on_gift_received(self, gift_msg: web_models.GiftMessage):
        self.gift_received.emit(gift_msg)

    def on_interact_received(self, interact_msg: web_models.InteractWordV2Message):
        self.interact_received.emit(interact_msg)

    def add_message(self, message):
        """通用添加消息方法"""
        item_widget = DanmakuItemWidget(message)
        
        # Pre-calculate height based on current viewport width
        width = self.danmaku_list.viewport().width()
        if width > 0:
            item_widget.setFixedWidth(width)
            size = item_widget.sizeHint()
            item_widget.setMinimumWidth(0)
            item_widget.setMaximumWidth(16777215)
        else:
            size = item_widget.sizeHint()

        item = QListWidgetItem()
        item.setSizeHint(size)
        
        self.danmaku_list.addItem(item)
        self.danmaku_list.setItemWidget(item, item_widget)
        self.danmaku_list.scrollToBottom()

        if self.danmaku_list.count() > 500:
            self.danmaku_list.takeItem(0)

    def open_qr_login(self):
        """打开扫码登录窗口"""
        dialog = QRLoginDialog(self)
        dialog.login_success.connect(self.on_login_success)
        dialog.exec()

    def on_login_success(self):
        """登录成功，提醒用户重连"""
        self.tray_icon.showMessage(
            "登录成功", 
            "B站账号已登录，将在下次连接时生效。", 
            QSystemTrayIcon.MessageIcon.Information, 
            2000
        )
        self.add_system_message("登录成功！请断开并重新连接以应用新的登录信息。")
        
        # 自动重连逻辑 (如果已连接)
        if self.danmaku_client and self.danmaku_client.session:
            # 简单处理：提示用户
            pass

    def on_login_failed(self, msg: str):
        """登录失效回调"""
        self.tray_icon.showMessage(
            "登录失效", 
            msg, 
            QSystemTrayIcon.MessageIcon.Warning, 
            5000
        )
        self.add_system_message(msg, "error")

    @qasync.asyncClose
    async def closeEvent(self, event: QCloseEvent):
        if self.danmaku_client:
            await self.danmaku_client.stop()
        event.accept()



import os
import json
from pathlib import Path
from typing import Dict, Any

def get_config_path() -> Path:
    """获取配置文件路径 (遵循XDG规范)"""
    xdg_config_home = os.environ.get('XDG_CONFIG_HOME') or os.path.expanduser('~/.config')
    config_dir = Path(xdg_config_home) / 'bilihud'
    config_dir.mkdir(parents=True, exist_ok=True)
    return config_dir / 'config.json'

def load_config() -> Dict[str, Any]:
    """加载配置"""
    config_path = get_config_path()
    if not config_path.exists():
        return {}
    
    try:
        with open(config_path, 'r', encoding='utf-8') as f:
            return json.load(f)
    except Exception as e:
        print(f"Failed to load config: {e}")
        return {}

def save_config(data: Dict[str, Any]) -> bool:
    """保存配置"""
    try:
        config_path = get_config_path()
        
        # 读取现有配置以进行合并，防止覆盖其他配置项
        current_config = load_config()
        current_config.update(data)
        
        with open(config_path, 'w', encoding='utf-8') as f:
            json.dump(current_config, f, indent=4, ensure_ascii=False)
        return True
    except Exception as e:
        print(f"Failed to save config: {e}")
        return False

def validate_room_id(room_id_str: str) -> bool:
    """
    验证直播间ID是否有效

    Args:
        room_id_str: 直播间ID字符串

    Returns:
        bool: 如果有效返回True，否则返回False
    """
    try:
        room_id = int(room_id_str)
        return room_id > 0
    except ValueError:
        return False


def format_danmaku_message(danmaku_msg) -> str:
    """
    格式化弹幕消息用于显示

    Args:
        danmaku_msg: 弹幕消息对象

    Returns:
        str: 格式化后的弹幕消息
    """
    return f"{danmaku_msg.uname}: {danmaku_msg.msg}"



# -*- coding: utf-8 -*-
import asyncio
import http.cookies
from typing import Optional, Callable
import aiohttp
import blivedm
import blivedm.models.web as web_models


class DanmakuClient:
    """
    弹幕客户端，用于获取B站直播弹幕
    """

    def __init__(self, room_id: int, sessdata: str = ''):
        self.room_id = room_id
        self.sessdata = sessdata
        self.session: Optional[aiohttp.ClientSession] = None
        self.client: Optional[blivedm.BLiveClient] = None
        self.handler: Optional[DanmakuHandler] = None
        self.on_danmaku_received: Optional[Callable[[web_models.DanmakuMessage], None]] = None
        self.on_gift_received: Optional[Callable[[web_models.GiftMessage], None]] = None
        self.on_interact_received: Optional[Callable[[web_models.InteractWordV2Message], None]] = None
        self.on_login_failed: Optional[Callable[[str], None]] = None # callback(message)

    def set_danmaku_callback(self, callback: Callable[[web_models.DanmakuMessage], None]):
        """设置弹幕接收回调函数"""
        self.on_danmaku_received = callback

    def set_gift_callback(self, callback: Callable[[web_models.GiftMessage], None]):
        """设置礼物接收回调函数"""
        self.on_gift_received = callback

    def set_interact_callback(self, callback: Callable[[web_models.InteractWordV2Message], None]):
        """设置互动接收回调函数 (进房/关注)"""
        self.on_interact_received = callback

    def set_login_failed_callback(self, callback: Callable[[str], None]):
        """设置登录失效回调"""
        self.on_login_failed = callback

    async def start(self):
        """在事件循环中启动弹幕客户端"""
        # 初始化session
        cookies = http.cookies.SimpleCookie()
        
        # 尝试优先从Keyring加载Cookies
        from .auth import AuthManager
        auth_manager = AuthManager()
        saved_cookies = auth_manager.load_cookies()
        
        loaded_from_keyring = False
        if saved_cookies:
            # 验证Cookie有效性
            is_valid = await auth_manager.validate_session(saved_cookies)
            if is_valid:
                for k, v in saved_cookies.items():
                    cookies[k] = v
                print("Successfully loaded validated cookies from Keyring")
                loaded_from_keyring = True
            else:
                print("Keyring cookies expired")
                if self.on_login_failed:
                    self.on_login_failed("本地保存的登录信息已失效，请重新登录")

        # 如果没有有效的Keyring Cookies，尝试从浏览器加载
        if not loaded_from_keyring:
            browser_cookies = load_bilibili_cookies()
            if browser_cookies:
                # print("Successfully loaded cookies from browser")
                # 将http.cookiejar.CookieJar转换为SimpleCookie或简单dict
                for cookie in browser_cookies:
                    cookies[cookie.name] = cookie.value
        
        if self.sessdata:
            cookies['SESSDATA'] = self.sessdata
            
        # 确保domain设置正确
        if 'SESSDATA' in cookies:
            cookies['SESSDATA']['domain'] = 'bilibili.com'

        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36 Edg/120.0.0.0'
        }
        self.session = aiohttp.ClientSession(headers=headers)
        self.session.cookie_jar.update_cookies(cookies)

        # 创建客户端和处理器
        self.client = blivedm.BLiveClient(self.room_id, session=self.session)
        self.handler = DanmakuHandler()
        self.handler.set_danmaku_client(self)
        self.client.set_handler(self.handler)

        loop = asyncio.get_event_loop()
        loop.call_soon(self.client.start)
        # 启动客户端
        # self.client.start()
        # await asyncio.sleep(0.1)  # 让出控制权以启动客户端

    async def send_danmaku(self, message: str) -> tuple[bool, str]:
        """发送弹幕"""
        if not self.session or not message:
            return False, "会话未初始化或消息为空"
            
        url = 'https://api.live.bilibili.com/msg/send'
        
        # 从cookie中获取csrf token (bili_jct)
        csrf_token = ''
        for cookie in self.session.cookie_jar:
            if cookie.key == 'bili_jct':
                csrf_token = cookie.value
                break
                
        if not csrf_token:
            # print("Error: No csrf_token found in cookies")
            return False, "未找到CSRF Token，请重新连接或检查Cookie"
            
        data = {
            'bubble': '0',
            'msg': message,
            'color': '16777215',
            'mode': '1',
            'fontsize': '25',
            'rnd': str(int(asyncio.get_event_loop().time())),
            'roomid': self.room_id,
            'csrf': csrf_token,
            'csrf_token': csrf_token,
        }
        
        try:
            async with self.session.post(url, data=data) as res:
                if res.status != 200:
                    print(f"Send danmaku HTTP error: {res.status}")
                    return False, f"HTTP错误: {res.status}"
                json_data = await res.json()
                if json_data['code'] == 0:
                    return True, "发送成功"
                else:
                    print(f"Send danmaku failed: {json_data['message']}")
                    return False, f"发送失败: {json_data['message']}"
        except Exception as e:
            print(f"Send danmaku exception: {e}")
            return False, f"发送异常: {str(e)}"

    async def stop(self):
        """停止弹幕客户端"""
        if self.client:
            await self.client.stop_and_close()
        if self.session:
            await self.session.close()


class DanmakuHandler(blivedm.BaseHandler):
    """弹幕处理器"""

    def __init__(self):
        super().__init__()
        self.danmaku_client: Optional[DanmakuClient] = None

    def set_danmaku_client(self, client: DanmakuClient):
        self.danmaku_client = client

    def _on_danmaku(self, client: blivedm.BLiveClient, message: web_models.DanmakuMessage):
        """处理弹幕消息"""
        if self.danmaku_client and self.danmaku_client.on_danmaku_received:
            self.danmaku_client.on_danmaku_received(message)

    def _on_gift(self, client: blivedm.BLiveClient, message: web_models.GiftMessage):
        """处理礼物消息"""
        if self.danmaku_client and self.danmaku_client.on_gift_received:
            self.danmaku_client.on_gift_received(message)
            
    def _on_interact_word_v2(self, client: blivedm.BLiveClient, message: web_models.InteractWordV2Message):
        """处理进入房间/关注"""
        if self.danmaku_client and self.danmaku_client.on_interact_received:
            self.danmaku_client.on_interact_received(message)

    def _on_super_chat(self, client: blivedm.BLiveClient, message: web_models.SuperChatMessage):
        """处理醒目留言"""
        # 可以在这里处理醒目留言
        pass


def load_bilibili_cookies():
    """尝试从浏览器加载B站Cookies"""
    try:
        import browser_cookie3
        # 尝试从Chrome加载
        # print("Attempting to load cookies from Chrome...")
        cj = browser_cookie3.chrome(domain_name='.bilibili.com')
        return cj
    except Exception as e:
        print(f"Chrome cookies failed: {e}")
        try:
            import browser_cookie3
            # 尝试从Edge加载
            cj = browser_cookie3.edge(domain_name='.bilibili.com')
            return cj
        except Exception as e:
            print(f"Edge cookies failed: {e}")
            try:
                import browser_cookie3
                # 尝试从Firefox加载
                cj = browser_cookie3.firefox(domain_name='.bilibili.com')
                return cj
            except Exception as e:
                print(f"Firefox cookies failed: {e}")
                return None






"""Top-level package for bilihud."""

import sys
import os

# 优先查找包内的 vendor (安装模式)
_local_vendor = os.path.join(os.path.dirname(__file__), 'vendor')
# 其次查找项目根目录的 vendor (开发模式)
# 开发模式结构: src/bilihud/__init__.py -> ../../vendor/blivedm
_dev_vendor = os.path.join(os.path.dirname(__file__), '..', '..', 'vendor', 'blivedm')

if os.path.exists(os.path.join(_local_vendor, 'blivedm')):
    if _local_vendor not in sys.path:
        sys.path.insert(0, _local_vendor)
elif os.path.exists(_dev_vendor):
    if _dev_vendor not in sys.path:
        sys.path.insert(0, _dev_vendor)




# -*- coding: utf-8 -*-
import dataclasses
import enum
from typing import *

import pure_protobuf.annotations as pb_anno
import pure_protobuf.message as pb_msg

try:
    Annotated
except NameError:
    from typing_extensions import Annotated  # Python < 3.9


class InteractWordV2MsgType(enum.IntEnum):
    Unknown = 0
    EnterRoom = 1
    Follow = 2
    ShareRoom = 3


@dataclasses.dataclass
class InteractWordV2UserBaseInfo(pb_msg.BaseMessage):
    face: Annotated[str, pb_anno.Field(2)] = ''


@dataclasses.dataclass
class InteractWordV2UserInfo(pb_msg.BaseMessage):
    base: Annotated[InteractWordV2UserBaseInfo, pb_anno.Field(2)] = dataclasses.field(default_factory=InteractWordV2UserBaseInfo)


@dataclasses.dataclass
class InteractWordV2(pb_msg.BaseMessage):
    uid: Annotated[int, pb_anno.Field(1)] = 0
    uname: Annotated[str, pb_anno.Field(2)] = ''
    # 为了防止加新枚举后不兼容，还是用int了
    # msg_type: Annotated[InteractWordV2MsgType, pb_anno.Field(5)] = InteractWordV2MsgType.Unknown
    msg_type: Annotated[int, pb_anno.Field(5)] = 0
    timestamp: Annotated[int, pb_anno.Field(7)] = 0
    uinfo: Annotated[InteractWordV2UserInfo, pb_anno.Field(22)] = dataclasses.field(default_factory=InteractWordV2UserInfo)



# -*- coding: utf-8 -*-
import base64
import dataclasses
import json
from typing import *

from . import pb

__all__ = (
    'HeartbeatMessage',
    'DanmakuMessage',
    'GiftMessage',
    'GuardBuyMessage',
    'SuperChatMessage',
    'SuperChatDeleteMessage',
)


@dataclasses.dataclass
class HeartbeatMessage:
    """
    心跳消息
    """

    popularity: int = 0
    """人气值，已废弃"""

    @classmethod
    def from_command(cls, data: dict):
        return cls(
            popularity=data['popularity'],
        )


@dataclasses.dataclass
class DanmakuMessage:
    """
    弹幕消息
    """

    mode: int = 0
    """弹幕显示模式（滚动、顶部、底部）"""
    font_size: int = 0
    """字体尺寸"""
    color: int = 0
    """颜色"""
    timestamp: int = 0
    """时间戳（毫秒）"""
    rnd: int = 0
    """随机数，前端叫作弹幕ID，可能是去重用的"""
    uid_crc32: str = ''
    """用户ID文本的CRC32"""
    msg_type: int = 0
    """是否礼物弹幕（节奏风暴）"""
    bubble: int = 0
    """右侧评论栏气泡"""
    dm_type: int = 0
    """弹幕类型，0文本，1表情，2语音"""
    emoticon_options: Union[dict, str] = ''
    """表情参数"""
    voice_config: Union[dict, str] = ''
    """语音参数"""
    mode_info: dict = dataclasses.field(default_factory=dict)
    """一些附加参数"""
    is_mirror: bool = False
    """是否跨房弹幕"""

    msg: str = ''
    """弹幕内容"""

    uid: int = 0
    """用户ID"""
    uname: str = ''
    """用户名"""
    face: str = ''
    """用户头像URL"""
    admin: int = 0
    """是否房管"""
    vip: int = 0
    """是否月费老爷"""
    svip: int = 0
    """是否年费老爷"""
    urank: int = 0
    """用户身份，用来判断是否正式会员，猜测非正式会员为5000，正式会员为10000"""
    mobile_verify: int = 0
    """是否绑定手机"""
    uname_color: str = ''
    """用户名颜色"""

    medal_level: int = 0
    """勋章等级"""
    medal_name: str = ''
    """勋章名"""
    runame: str = ''
    """勋章房间主播名"""
    medal_room_id: int = 0
    """勋章房间ID"""
    mcolor: int = 0
    """勋章颜色"""
    special_medal: str = ''
    """特殊勋章"""

    user_level: int = 0
    """用户等级"""
    ulevel_color: int = 0
    """用户等级颜色"""
    ulevel_rank: str = ''
    """用户等级排名，>50000时为'>50000'"""

    old_title: str = ''
    """旧头衔"""
    title: str = ''
    """头衔"""

    privilege_type: int = 0
    """舰队类型，0非舰队，1总督，2提督，3舰长"""

    wealth_level: int = 0
    """荣耀等级"""

    @classmethod
    def from_command(cls, info: list):
        mode_info = info[0][15]
        try:
            face = mode_info['user']['base']['face']
        except (TypeError, KeyError):
            face = ''

        if len(info[3]) != 0:
            medal_level = info[3][0]
            medal_name = info[3][1]
            runame = info[3][2]
            medal_room_id = info[3][3]
            mcolor = info[3][4]
            special_medal = info[3][5]
        else:
            medal_level = 0
            medal_name = ''
            runame = ''
            medal_room_id = 0
            mcolor = 0
            special_medal = 0

        if len(info[5]) != 0:
            old_title = info[5][0]
            title = info[5][1]
        else:
            old_title = ''
            title = ''

        return cls(
            mode=info[0][1],
            font_size=info[0][2],
            color=info[0][3],
            timestamp=info[0][4],
            rnd=info[0][5],
            uid_crc32=info[0][7],
            msg_type=info[0][9],
            bubble=info[0][10],
            dm_type=info[0][12],
            emoticon_options=info[0][13],
            voice_config=info[0][14],
            mode_info=mode_info,

            msg=info[1],

            uid=info[2][0],
            uname=info[2][1],
            face=face,
            admin=info[2][2],
            vip=info[2][3],
            svip=info[2][4],
            urank=info[2][5],
            mobile_verify=info[2][6],
            uname_color=info[2][7],

            medal_level=medal_level,
            medal_name=medal_name,
            runame=runame,
            medal_room_id=medal_room_id,
            mcolor=mcolor,
            special_medal=special_medal,

            user_level=info[4][0],
            ulevel_color=info[4][2],
            ulevel_rank=info[4][3],

            old_title=old_title,
            title=title,

            privilege_type=info[7],

            wealth_level=info[16][0],
        )

    @property
    def emoticon_options_dict(self) -> dict:
        """
        示例：

        ```
        {'bulge_display': 0, 'emoticon_unique': 'official_13', 'height': 60, 'in_player_area': 1, 'is_dynamic': 1,
         'url': 'https://i0.hdslb.com/bfs/live/a98e35996545509188fe4d24bd1a56518ea5af48.png', 'width': 183}
         ```
        """
        if isinstance(self.emoticon_options, dict):
            return self.emoticon_options
        try:
            return json.loads(self.emoticon_options)
        except (json.JSONDecodeError, TypeError):
            return {}

    @property
    def voice_config_dict(self) -> dict:
        """
        示例：

        ```
        {'voice_url': 'https%3A%2F%2Fboss.hdslb.com%2Flive-dm-voice%2Fb5b26e48b556915cbf3312a59d3bb2561627725945.wav
         %3FX-Amz-Algorithm%3DAWS4-HMAC-SHA256%26X-Amz-Credential%3D2663ba902868f12f%252F20210731%252Fshjd%252Fs3%25
         2Faws4_request%26X-Amz-Date%3D20210731T100545Z%26X-Amz-Expires%3D600000%26X-Amz-SignedHeaders%3Dhost%26
         X-Amz-Signature%3D114e7cb5ac91c72e231c26d8ca211e53914722f36309b861a6409ffb20f07ab8',
         'file_format': 'wav', 'text': '汤，下午好。', 'file_duration': 1}
         ```
        """
        if isinstance(self.voice_config, dict):
            return self.voice_config
        try:
            return json.loads(self.voice_config)
        except (json.JSONDecodeError, TypeError):
            return {}

    @property
    def extra_dict(self) -> dict:
        """
        示例：

        ```
        {'send_from_me': False, 'mode': 0, 'color': 14893055, 'dm_type': 0, 'font_size': 25, 'player_mode': 4,
        'show_player_type': 0, 'content': '确实', 'user_hash': '2904574201', 'emoticon_unique': '', 'bulge_display': 0,
        'recommend_score': 5, 'main_state_dm_color': '', 'objective_state_dm_color': '', 'direction': 0,
        'pk_direction': 0, 'quartet_direction': 0, 'anniversary_crowd': 0, 'yeah_space_type': '', 'yeah_space_url': '',
        'jump_to_url': '', 'space_type': '', 'space_url': '', 'animation': {}, 'emots': None, 'is_audited': False,
        'id_str': '6fa9959ab8feabcd1b337aa5066768334027', 'icon': None, 'show_reply': True, 'reply_mid': 0,
        'reply_uname': '', 'reply_uname_color': '', 'reply_is_mystery': False, 'reply_type_enum': 0, 'hit_combo': 0,
        'esports_jump_url': ''}
        ```
        """
        try:
            extra = self.mode_info['extra']
            if isinstance(extra, dict):
                return extra
            return json.loads(extra)
        except (KeyError, json.JSONDecodeError, TypeError):
            return {}


@dataclasses.dataclass
class GiftMessage:
    """
    礼物消息
    """

    gift_name: str = ''
    """礼物名"""
    num: int = 0
    """数量"""
    uname: str = ''
    """用户名"""
    face: str = ''
    """用户头像URL"""
    guard_level: int = 0
    """舰队等级，0非舰队，1总督，2提督，3舰长"""
    uid: int = 0
    """用户ID"""
    timestamp: int = 0
    """时间戳"""
    gift_id: int = 0
    """礼物ID"""
    gift_type: int = 0
    """礼物类型（未知）"""
    gift_img_basic: str = ''
    """图标URL"""
    action: str = ''
    """目前遇到的有'喂食'、'赠送'"""
    price: int = 0
    """礼物单价瓜子数"""
    rnd: str = ''
    """随机数，可能是去重用的。有时是时间戳+去重ID，有时是UUID"""
    coin_type: str = ''
    """瓜子类型，'silver'或'gold'，1000金瓜子 = 1元"""
    total_coin: int = 0
    """总瓜子数"""
    tid: str = ''
    """可能是事务ID，有时和rnd相同"""
    medal_level: int = 0
    """勋章等级"""
    medal_name: str = ''
    """勋章名"""
    medal_room_id: int = 0
    """勋章房间ID，未登录时是0"""
    medal_ruid: int = 0
    """勋章主播ID"""

    @classmethod
    def from_command(cls, data: dict):
        medal_info = data.get('medal_info', None)
        if medal_info is not None:
            medal_level = medal_info['medal_level']
            medal_name = medal_info['medal_name']
            medal_room_id = medal_info['anchor_roomid']
            medal_ruid = medal_info['target_id']
        else:
            medal_level = 0
            medal_name = ''
            medal_room_id = 0
            medal_ruid = 0

        return cls(
            gift_name=data['giftName'],
            num=data['num'],
            uname=data['uname'],
            face=data['face'],
            guard_level=data['guard_level'],
            uid=data['uid'],
            timestamp=data['timestamp'],
            gift_id=data['giftId'],
            gift_type=data['giftType'],
            gift_img_basic=data['gift_info']['img_basic'],
            action=data['action'],
            price=data['price'],
            rnd=data['rnd'],
            coin_type=data['coin_type'],
            total_coin=data['total_coin'],
            tid=data['tid'],
            medal_level=medal_level,
            medal_name=medal_name,
            medal_room_id=medal_room_id,
            medal_ruid=medal_ruid,
        )


@dataclasses.dataclass
class GuardBuyMessage:
    """
    上舰消息
    """

    uid: int = 0
    """用户ID"""
    username: str = ''
    """用户名"""
    guard_level: int = 0
    """舰队等级，0非舰队，1总督，2提督，3舰长"""
    num: int = 0  # 可以理解为礼物数量？
    """数量"""
    price: int = 0
    """单价金瓜子数"""
    gift_id: int = 0
    """礼物ID"""
    gift_name: str = ''
    """礼物名"""
    start_time: int = 0
    """开始时间戳，和结束时间戳相同"""
    end_time: int = 0
    """结束时间戳，和开始时间戳相同"""

    @classmethod
    def from_command(cls, data: dict):
        return cls(
            uid=data['uid'],
            username=data['username'],
            guard_level=data['guard_level'],
            num=data['num'],
            price=data['price'],
            gift_id=data['gift_id'],
            gift_name=data['gift_name'],
            start_time=data['start_time'],
            end_time=data['end_time'],
        )


@dataclasses.dataclass
class UserToastV2Message:
    """
    另一个上舰消息，包含的数据更多
    """

    uid: int = 0
    """用户ID"""
    username: str = ''
    """用户名"""
    guard_level: int = 0
    """舰队等级，0非舰队，1总督，2提督，3舰长"""
    num: int = 0  # 可以理解为礼物数量？
    """数量"""
    price: int = 0
    """单价金瓜子数"""
    unit: str = ''
    """单位，根据开放平台的文档，正常单位为“月”，如为其他内容，无视`guard_num`以本字段内容为准，例如`*3天`"""
    gift_id: int = 0
    """礼物ID"""
    start_time: int = 0
    """开始时间戳，和结束时间戳相同"""
    end_time: int = 0
    """结束时间戳，和开始时间戳相同"""
    source: int = 0
    """猜测0是付费买的，2是赠送的。现在上舰时会先发一条0的消息再发一条2的消息，官方的评论栏不会显示2的消息"""
    toast_msg: str = ''
    """提示信息（"<%XXX%> 在主播XXX的直播间开通了舰长，今天是TA陪伴主播的第XXX天"）"""

    @classmethod
    def from_command(cls, data: dict):
        sender_info = data['sender_uinfo']
        guard_info = data['guard_info']
        pay_info = data['pay_info']
        gift_info = data['gift_info']
        option = data['option']
        return cls(
            uid=sender_info['uid'],
            username=sender_info['base']['name'],
            guard_level=guard_info['guard_level'],
            num=pay_info['num'],
            price=pay_info['price'],
            unit=pay_info['unit'],
            gift_id=gift_info['gift_id'],
            start_time=guard_info['start_time'],
            end_time=guard_info['end_time'],
            source=option['source'],
            toast_msg=data['toast_msg'],
        )


@dataclasses.dataclass
class SuperChatMessage:
    """
    醒目留言消息
    """

    price: int = 0
    """价格（人民币）"""
    message: str = ''
    """消息"""
    message_trans: str = ''
    """消息日文翻译"""
    start_time: int = 0
    """开始时间戳"""
    end_time: int = 0
    """结束时间戳"""
    time: int = 0
    """剩余时间（约等于 结束时间戳 - 开始时间戳）"""
    id: int = 0
    """醒目留言ID，删除时用"""
    gift_id: int = 0
    """礼物ID"""
    gift_name: str = ''
    """礼物名"""
    uid: int = 0
    """用户ID"""
    uname: str = ''
    """用户名"""
    face: str = ''
    """用户头像URL"""
    guard_level: int = 0
    """舰队等级，0非舰队，1总督，2提督，3舰长"""
    user_level: int = 0
    """用户等级"""
    background_bottom_color: str = ''
    """底部背景色，'#rrggbb'"""
    background_color: str = ''
    """背景色，'#rrggbb'"""
    background_icon: str = ''
    """背景图标"""
    background_image: str = ''
    """背景图URL"""
    background_price_color: str = ''
    """背景价格颜色，'#rrggbb'"""
    medal_level: int = 0
    """勋章等级"""
    medal_name: str = ''
    """勋章名"""
    medal_room_id: int = 0
    """勋章房间ID"""
    medal_ruid: int = 0
    """勋章主播ID"""

    @classmethod
    def from_command(cls, data: dict):
        medal_info = data.get('medal_info', None)
        if medal_info is not None:
            medal_level = medal_info['medal_level']
            medal_name = medal_info['medal_name']
            medal_room_id = medal_info['anchor_roomid']
            medal_ruid = medal_info['target_id']
        else:
            medal_level = 0
            medal_name = ''
            medal_room_id = 0
            medal_ruid = 0

        return cls(
            price=data['price'],
            message=data['message'],
            message_trans=data['message_trans'],
            start_time=data['start_time'],
            end_time=data['end_time'],
            time=data['time'],
            id=data['id'],
            gift_id=data['gift']['gift_id'],
            gift_name=data['gift']['gift_name'],
            uid=data['uid'],
            uname=data['user_info']['uname'],
            face=data['user_info']['face'],
            guard_level=data['user_info']['guard_level'],
            user_level=data['user_info']['user_level'],
            background_bottom_color=data['background_bottom_color'],
            background_color=data['background_color'],
            background_icon=data['background_icon'],
            background_image=data['background_image'],
            background_price_color=data['background_price_color'],
            medal_level=medal_level,
            medal_name=medal_name,
            medal_room_id=medal_room_id,
            medal_ruid=medal_ruid,
        )


@dataclasses.dataclass
class SuperChatDeleteMessage:
    """
    删除醒目留言消息
    """

    ids: List[int] = dataclasses.field(default_factory=list)
    """醒目留言ID数组"""

    @classmethod
    def from_command(cls, data: dict):
        return cls(
            ids=data['ids'],
        )


@dataclasses.dataclass
class InteractWordV2Message:
    """
    进入房间、关注主播等互动消息
    """

    uid: int = 0
    """用户ID"""
    username: str = ''
    """用户名"""
    face: str = ''
    """用户头像URL"""
    timestamp: int = 0
    """时间戳"""
    msg_type: int = 0
    """`{1: '进入', 2: '关注了', 3: '分享了', 4: '特别关注了', 5: '互粉了', 6: '为主播点赞了'}`"""

    @classmethod
    def from_command(cls, data: dict):
        proto = pb.InteractWordV2.loads(base64.b64decode(data['pb']))
        return cls(
            uid=proto.uid,
            username=proto.uname,
            face=proto.uinfo.base.face,
            timestamp=proto.timestamp,
            msg_type=proto.msg_type,
        )
